@@ -23,7 +23,7 @@
 (use-modules (nyacc lex))
 (use-modules (nyacc util))
 (use-modules (sxml fold))
-(use-modules (sxml xpath))
+(use-modules ((sxml xpath) #:hide (filter)))
 (use-modules ((srfi srfi-1) #:select (last fold-right)))
 (use-modules (srfi srfi-11))            ; let-values
 (use-modules (rnrs arithmetic bitwise))
@@ -41,6 +41,10 @@
 (define (pperr sx) (pretty-print sx (cep) #:per-line-prefix "  "))
 (define (ppe99 sx) (pretty-print-c99 sx (cep) #:per-line-prefix "  "))
 
+(define (pp99s exp)
+  (call-with-output-string (lambda (port) (pretty-print-c99 exp port))))
+
+
 (define *cpp-defs* (get-gcc-cpp-defs))
 (define *inc-dirs* (get-gcc-inc-dirs))
 (define *inc-help* c99-def-help)
@@ -371,30 +375,6 @@
     (pp (find-offsets mtail udict))
     0))
 
-(when #f
-  (let* ((code
-          (string-append
-           "typedef struct { int p; int q; } foo_t;\n"
-           "typedef struct { int r; foo_t s; foo_t *t; } bar_t;\n"
-           "typedef struct { int u; bar_t v; bar_t *w; } baz_t;\n"
-           "baz_t f;\n"
-           ))
-         (tree (parse-string code))
-         (udict (c99-trans-unit->udict tree))
-         (udecl (udict-ref udict "f"))
-         (t-exp `(d-sel (ident "p")
-                        (i-sel (ident "t")
-                               (d-sel (ident "v")
-                                      (ident "f")))))
-         (t-exp `(i-sel (ident "r")
-                        (d-sel (ident "w")
-                               (ident "f"))))
-         #;(t-exp `(d-sel (ident "u") (ident "f")))
-         )
-    ;;(pp t-exp) (pp99 t-exp) (sf "\n")
-    (pp (eval-typeof-expr t-exp udict))
-    0))
-
 (when #f
   (let* ((code
           (string-append
@@ -539,4 +519,24 @@ typedef struct _GObjectClass {
     (pp99 tree)
     ))
 
+(when #f
+  (let* ((file "/usr/lib/gcc/x86_64-linux-gnu/12/include/stddef.h")
+         (file "/usr/include/glib-2.0/glib/gmacros.h")
+         (file "/usr/include/glib-2.0/glib.h")
+         (ginc '("/usr/include/glib-2.0/"
+                 "/usr/lib/x86_64-linux-gnu/glib-2.0/include"))
+         (tree (parse-file
+                file
+                #:inc-dirs (append ginc (get-sys-inc-dirs))))
+         )
+    #f))
+
+(when #f ;; bug in 1.08
+  (let* ((make-cpi (@@ (nyacc lang c99 parser) make-cpi))
+         (cpi-defs (@@ (nyacc lang c99 parser) cpi-defs))
+         (defs '("YY=#f" "ZZ=(-1)"))
+         (info (make-cpi #f #f defs '(".") '())))
+    (pp (cpi-defs info))
+    #f))
+
 ;; --- last line ---
